require 'miniserver/request'
require 'miniserver/response'
require 'http/parser'
require 'time'

module Miniserver
  class Connection < EM::Connection
    attr_accessor :app

    def post_init
      @request  = Request.new
      @response = Response.new
      @parser   = Http::Parser.new(self)
    end

    def on_headers_complete(headers)
      @request.headers = headers
      @request.parse_header(@parser, self)
    end

    def on_body(chunk)
      @request.body << chunk
    end

    def on_message_complete
      begin
        Rack::Handler::Miniserver.new(app).process(@request, @response)
        send_response!
      rescue
        Miniserver.logger.error("!! Unexpected error while processing request: #{$!.message}")
      ensure
        close_connection_after_writing
      end
    end

    def receive_data(data)
      @parser << data
    end

    def send_response!
      send_status_line
      send_headers
      send_data(BLANK_LINE)
      send_body
    end

    def send_status_line
      send_data("#{HTTP_VERSION} #{@response.status} #{HTTP_CODE[@response.status]}#{BLANK_LINE}")
    end

    def send_headers
      headers = String.new

      @response.headers[SERVER] = MINISERVER
      @response.headers[DATE] = Time.now.httpdate

      @response.headers.each do |name, value|
        if value.is_a?(String)
          value.split(NEWLINE).each do |v|
            headers.concat("#{name}#{COLON} #{v}#{BLANK_LINE}")
          end
        else
          headers.concat("#{name}#{COLON} #{value}#{BLANK_LINE}")
        end
      end

      send_data(headers)
    end

    def send_body
      begin 
        @response.body.each { |chunk| send_data(chunk) }
      ensure
        @response.body.close if @response.body.respond_to?(:close)
      end
    end
  end
end
