module Miniserver
  class Runner
    attr_reader :app, :options

    def initialize(options)
      @options = options
    end

    def status
      status = if File.exist?(options[:pid])
                 pid = File.open(options[:pid], 'r').read.to_i

                 if Process.running?(pid)
                   :Started
                 else
                   :Stopped
                 end
               else
                 :Stopped
               end

      puts status
    end

    def start
      if File.exist?(options[:pid])
        pid = File.open(options[:pid], 'r').read.to_i

        if Process.running?(pid)
          raise "Miniserver has started in pid #{pid}."
        else
          File.delete(options[:pid])
        end
      end

      require 'rack/builder'

      ENV["RACK_ENV"] = options[:environment]

      @app = Rack::Builder.new_from_string(File.read(options[:rackup]))

      Server.new(app, options).start
    end

    def stop
      if File.exist?(options[:pid])
        pid = File.open(options[:pid], 'r').read.to_i
        Process.kill(:INT, pid)
        File.delete(options[:pid])
      else
        raise 'Miniserver is not running.'
      end
    end

    def restart
      if File.exist?(options[:pid])
        pid = File.open(options[:pid], 'r').read.to_i
        Process.kill(:HUP, pid)
      else
        raise 'Miniserver is not running.'
      end
    end
  end
end
