require 'eventmachine'
require 'miniserver/connection'
require 'miniserver/logger'
require 'fileutils'

module Miniserver
  class Server
    attr_reader :app, :options, :script
  
    def initialize(app, options={})
      @app  = app
      @options = options
      @script = $PROGRAM_NAME

      FileUtils.mkdir_p(File.dirname(options[:log]))
      Miniserver.logger = Logger.new(options[:log])
    end

    def start
      Miniserver.logger.info "=> #{Time.now.asctime}"
      Miniserver.logger.info "=> Starting Miniserver #{Miniserver::VERSION}"
      Miniserver.logger.info "=> Listening on #{options[:host]}:#{options[:port]}"

      if options[:daemonize]
        daemonize_app
      else
        puts "=> Miniserver #{Miniserver::VERSION}"
        puts "=> Listening on #{options[:host]}:#{options[:port]}"
        puts "=> CTRL+C to stop"
      end

      trap(:INT) { stop }
      trap(:TERM) { stop }
      trap(:HUP) { restart }

      EM.run do
        EM.start_server(options[:host], options[:port], Connection) do |connection|
          connection.app = app
        end
      end
    end

    def daemonize_app
      Process.daemon(true)

      $0 = "Miniserver #{Miniserver::VERSION} (#{options[:host]}:#{options[:port]})"

      FileUtils.mkdir_p(File.dirname(options[:pid]))

      File.open(options[:pid], "w") do |f|
        f.puts(Process.pid)
      end
    end

    def stop
      EM.stop
    end

    def restart
      stop

      Command.restart(script, options)
    end
  end
end
